﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Wizard.WizardControl.Models;
using System.Collections.ObjectModel;
using CoffeeLibrary;
using System.ComponentModel;

namespace Wizard.CoffeeModel
{

    public class ExtrasStepViewModel : WizardStepViewModelBase<CupOfCoffee>
    {

        ReadOnlyCollection<OptionViewModel<Flavorings>> _availableFlavorings;
        ReadOnlyCollection<OptionViewModel<Temperature>> _availableTemperatures;

        public ExtrasStepViewModel( CupOfCoffee cupOfCoffee )
            : base( cupOfCoffee )
        {
        }

        /// <summary>
        /// Returns a read-only collection of all flavorings that the user can select.
        /// </summary>
        public ReadOnlyCollection<OptionViewModel<Flavorings>> AvailableFlavorings
        {
            get
            {
                if ( _availableFlavorings == null )
                    this.CreateAvailableFlavorings();

                return _availableFlavorings;
            }
        }

        void CreateAvailableFlavorings()
        {
            List<OptionViewModel<Flavorings>> list = new List<OptionViewModel<Flavorings>>();
            list.Add( new OptionViewModel<Flavorings>( Flavorings.Caramel ) );
            list.Add( new OptionViewModel<Flavorings>( Flavorings.Gingerbread ) );
            list.Add( new OptionViewModel<Flavorings>( Flavorings.Hazelnut ) );
            list.Add( new OptionViewModel<Flavorings>( Flavorings.IrishCream ) );
            list.Add( new OptionViewModel<Flavorings>( Flavorings.Pumpkin ) );
            list.Add( new OptionViewModel<Flavorings>( Flavorings.Vanilla ) );

            foreach ( OptionViewModel<Flavorings> option in list )
                option.PropertyChanged += this.OnFlavoringOptionPropertyChanged;

            list.Sort();

            _availableFlavorings = new ReadOnlyCollection<OptionViewModel<Flavorings>>( list );
        }

        void OnFlavoringOptionPropertyChanged( object sender, PropertyChangedEventArgs e )
        {
            // Since the user can select more than one additional flavoring to add to their
            // cup of coffee, we check all available flavorings and combine the enum values.

            Flavorings flavorings = Flavorings.None;

            foreach ( OptionViewModel<Flavorings> option in this.AvailableFlavorings )
                if ( option.IsSelected )
                    flavorings |= option.GetValue();

            base.BusinessObject.Flavorings = flavorings;
        }

        /// <summary>
        /// Returns a read-only collection of all coffee temperatures that the user can select.
        /// </summary>
        public ReadOnlyCollection<OptionViewModel<Temperature>> AvailableTemperatures
        {
            get
            {
                if ( _availableTemperatures == null )
                    this.CreateAvailableTemperatures();

                return _availableTemperatures;
            }
        }

        void CreateAvailableTemperatures()
        {
            List<OptionViewModel<Temperature>> list = new List<OptionViewModel<Temperature>>();
            list.Add( new OptionViewModel<Temperature>( Temperature.ExtraHot, 1 ) );
            list.Add( new OptionViewModel<Temperature>( Temperature.Normal, 0 ) );
            list.Add( new OptionViewModel<Temperature>( Temperature.WithIce, 2 ) );

            foreach ( OptionViewModel<Temperature> option in list )
            {
                // Select the default value.
                if ( option.GetValue() == base.BusinessObject.Temperature )
                    option.IsSelected = true;

                option.PropertyChanged += this.OnTemperatureOptionPropertyChanged;
            }

            list.Sort();

            _availableTemperatures = new ReadOnlyCollection<OptionViewModel<Temperature>>( list );
        }

        void OnTemperatureOptionPropertyChanged( object sender, PropertyChangedEventArgs e )
        {
            OptionViewModel<Temperature> option = sender as OptionViewModel<Temperature>;
            if ( option.IsSelected )
                base.BusinessObject.Temperature = option.GetValue();
        }

        public override string DisplayName
        {
            get { return "Extras"; }
        }

        public override bool IsValid()
        {
            return true;
        }

    }

}
