﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;
using Wizard.WizardControl.Models;
using CoffeeLibrary;

namespace Wizard.CoffeeModel
{

    public class SummaryStepViewModel : WizardStepViewModelBase<CupOfCoffee>
    {
        readonly ReadOnlyCollection<OptionViewModel<BeanType>> _availableBeanTypes;
        readonly RouteOptionGroupViewModel<DrinkSize> _availableDrinkSizes;
        readonly ReadOnlyCollection<OptionViewModel<Flavorings>> _availableFlavorings;
        readonly ReadOnlyCollection<OptionViewModel<Temperature>> _availableTemperatures;

        public SummaryStepViewModel(
            CupOfCoffee c,
            ReadOnlyCollection<OptionViewModel<BeanType>> availableBeanTypes,
            RouteOptionGroupViewModel<DrinkSize> availableDrinkSizes,
            ReadOnlyCollection<OptionViewModel<Flavorings>> availableFlavorings,
            ReadOnlyCollection<OptionViewModel<Temperature>> availableTemperatures )
            : base( c )
        {
            _availableBeanTypes = availableBeanTypes;
            _availableDrinkSizes = availableDrinkSizes;
            _availableFlavorings = availableFlavorings;
            _availableTemperatures = availableTemperatures;
        }

        #region Properties

        public string BeanType
        {
            get
            {
                var options = new List<IWizardOption>( _availableBeanTypes.Count );
                _availableBeanTypes.ToList().ForEach( bt => options.Add( (IWizardOption)bt ) );
                return GetSelectedOptionDisplayName( options );
            }
        }

        public override string DisplayName
        {
            get { return "Summary"; }
        }

        public string DrinkSize
        {
            get
            {
                var options = new List<IWizardOption>( _availableBeanTypes.Count );
                _availableDrinkSizes.OptionModels.ToList().ForEach( bt => options.Add( (IWizardOption)bt ) );
                return GetSelectedOptionDisplayName( options );
            }
        }

        public string Flavorings
        {
            get
            {
                List<String> flavoringNames =
                    ( from flavoringOption in _availableFlavorings
                      where flavoringOption.IsSelected
                      select flavoringOption.DisplayName ).ToList();

                string returnValue = null;

                for ( int i = 0; i < flavoringNames.Count; ++i )
                {
                    returnValue += flavoringNames[i];

                    if ( i < flavoringNames.Count - 1 )
                        returnValue += ", ";
                }

                return returnValue ?? "(none)";
            }
        }

        public string Temperature
        {
            get
            {
                var options = new List<IWizardOption>( _availableBeanTypes.Count );
                _availableTemperatures.ToList().ForEach( bt => options.Add( (IWizardOption)bt ) );
                return GetSelectedOptionDisplayName( options );
            }
        }

        #endregion // Properties

        #region Methods

        public override bool IsValid()
        {
            return true;
        }

        #endregion // Methods

        #region Private Helpers

        static string GetSelectedOptionDisplayName( List<IWizardOption> availableOptions )
        {
            if ( availableOptions == null )
                throw new ArgumentNullException( "availableOptions" );

            return ( from option in availableOptions
                     where option.IsSelected
                     select option.DisplayName ).FirstOrDefault();
        }

        #endregion // Private Helpers
    }

}
